// ZipTextureDemo
//
//	This is a simple program that shows how to load a texture from a .ZIP file.  It uses
//	DirectX9 (along with the D3DX utility library) and the Unzip code (based on zlib) 
//	by Gilles Vollant.   
//
//	Everything not copyrighted already is (C) 2003 Greg Stewart. :-)  Feel free to use
//	this code for whatever you want.

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <d3d9.h>
#include <d3dx9.h>
#include <d3dx9tex.h>
#include "unzip/unzip.h"

// SHORTCUTS
#define SAFE_RELEASE(x)		if(x) { x->Release(); x = 0; }

// GLOBALS
LPDIRECT3D9			g_pD3D			= 0;			// D3D object
LPDIRECT3DDEVICE9	g_pD3DDevice	= 0;			// D3D device

LPDIRECT3DTEXTURE9	g_pTexture		= 0;
LPD3DXSPRITE		g_pSprite		= 0;

// LOCAL FUNCTIONS
// windows
LRESULT CALLBACK	WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HWND				CreateMyWindow( HINSTANCE hInstance );
// directx
HRESULT				SetupDirectX( HWND hWnd );
VOID				ShutdownDirectX();
// general game
VOID				SetupGame();
VOID				RunGameCycle();
VOID				ShutdownGame();
// gfx utility
HRESULT				CreateTextureFromFileInZip( LPDIRECT3DDEVICE9 pDevice, 
											   LPCSTR szZip, LPCSTR szFile,
											   LPDIRECT3DTEXTURE9 *ppTexture );

// ======================
// WinMain()
//
//	Entry point.
// ======================
INT WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR, INT )
{
	HWND hWnd = CreateMyWindow( hInstance );

	if(! hWnd )
		return 0;

	SetupDirectX( hWnd );
	SetupGame();

	// begin message pump
	MSG msg;
	while( 1 )
	{
		if( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) )
		{
			if(msg.message == WM_QUIT )
				break;

			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else
		{
			RunGameCycle();
		}
	}

	ShutdownGame();
	ShutdownDirectX();
	return msg.wParam;
}


// =========================================
// SetupGame()
//
//	Initializes game stuff.
// =========================================
VOID SetupGame()
{
	D3DXCreateSprite( g_pD3DDevice, &g_pSprite );
	CreateTextureFromFileInZip( g_pD3DDevice, "board.zip", "board.bmp", &g_pTexture );
}

// =========================================
// ShutdownGame()
//
//	Cleans up memory/resources, etc.
// =========================================
VOID ShutdownGame()
{
	SAFE_RELEASE( g_pSprite );
	SAFE_RELEASE( g_pTexture );
}

// =========================================
// RunGameCycle()
//
//	Runs a single game cycle (frame)
// =========================================
VOID RunGameCycle()
{
	g_pD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB(20, 20, 200), 1.0f, 0 );
	g_pD3DDevice->BeginScene();

	g_pSprite->Draw( g_pTexture, NULL, NULL, NULL, 0.0f, NULL, 0xFFFFFFFF );

	g_pD3DDevice->EndScene();
	g_pD3DDevice->Present( NULL, NULL, NULL, NULL );
}


// ====================================================================================
// CreateTextureFromFileInZip( LPDIRECT3DDEVICE9, LPCSTR, LPCSTR, LPDIRECT3DTEXTURE9 *)
//
//	Loads a .ZIP file and reads a texture from 
//	a file inside.
//
//	NOTE:	You might want to add more (or better) error checking to this function to 
//			return something other than E_FAIL if the ZIP or the file inside it cannot 
//			be found.
//
//	Parameters:
//		pDevice			The D3D device.
//		szZip			The name of the .ZIP file to search.
//		szFile			The file to load texture from.
//		ppTexture		A pointer to a LPDIRECT3DTEXTURE9 where D3DX can store our
//						texture.  Remember to release it after you're done!
//
//	Return values:
//		S_OK			Everything went great.
//		E_FAIL			Something went wrong. :-)
//		E_OUTOFMEMORY	Our request to malloc was denied.
//
//
//	Other Notes:
//		When specifying a file to load from the zip, you may include a full path or a 
//		relative path.  Just make sure to use / slashes and not \.  However, you're
//		ZIP program may not store full paths, so you may want to take that into 
//		consideration.  Just specify the path as it appears in your ZIP program.
//
//		For example:
//
//			CreateTextureFromFileInZip( g_pD3DDevice, "board.zip", "art/board.bmp",
//										&g_pTexture );
// ====================================================================================
HRESULT	CreateTextureFromFileInZip( LPDIRECT3DDEVICE9 pDevice, LPCSTR szZip, LPCSTR szFile, 
								   LPDIRECT3DTEXTURE9 *ppTexture )
{
	// open ZIP
	unzFile zip = unzOpen( szZip );

	if( !zip )
		return E_FAIL;
	
	// locate the file and open it
	unzLocateFile( zip, szFile, 1 );
	
	if( unzOpenCurrentFile( zip ) != UNZ_OK )
		return E_FAIL;
	
	// find current file info (we are looking for uncompressed file size)
	unz_file_info info;
	unzGetCurrentFileInfo( zip, &info, NULL, 0, NULL, 0, NULL, 0 );
	
	// create a buffer big enough to hold uncompressed file in memory
	void *buffer = malloc( info.uncompressed_size );

	if( !buffer )
	{
		unzCloseCurrentFile( zip );
		unzClose( zip );

		return E_OUTOFMEMORY;
	}
	
	// load into memory
	unzReadCurrentFile( zip, buffer, info.uncompressed_size );
	
	unzCloseCurrentFile( zip );
	unzClose( zip );

	// use D3DX utility library to load our texture now.
	if( FAILED( D3DXCreateTextureFromFileInMemory( pDevice, buffer, info.uncompressed_size, 
		ppTexture ) ) )
	{
		free( buffer );
		return E_FAIL;
	}

	free( buffer );
	return S_OK;
}

// =========================================
// SetupDirectX()
//
//	Initializes everything DirectX related.
// =========================================

HRESULT SetupDirectX( HWND hWnd )
{
	g_pD3D = Direct3DCreate9( D3D_SDK_VERSION );

	if( !g_pD3D )
		return E_FAIL;

	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof( d3dpp ) );
	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;

	if( FAILED( g_pD3D->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, 
		D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &g_pD3DDevice ) ) )
	{
		return E_FAIL;
	}

	return S_OK;
}

// =========================================
// ShutdownDirectX()
//
//	Cleans up everything DirectX related.
// =========================================
VOID ShutdownDirectX()
{
	SAFE_RELEASE( g_pD3DDevice );
	SAFE_RELEASE( g_pD3D );
}

// ==============================
// CreateMyWindow()
//
//	Creates the application window.
// ==============================

HWND CreateMyWindow( HINSTANCE hInstance )
{	
	// create our class
	WNDCLASSEX wc = 
	{ 
		sizeof(WNDCLASSEX), CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW,
		WndProc, 0, 0, hInstance, LoadIcon( NULL, IDI_APPLICATION ),
		LoadCursor( NULL, IDC_ARROW ), (HBRUSH) GetStockObject( BLACK_BRUSH ),
		NULL, "SAMPLE", LoadIcon( NULL, IDI_APPLICATION )
	};

	// register window class

	if( !RegisterClassEx( &wc ) )
		return NULL;

	// Create window 
	HWND hWnd = CreateWindowEx( NULL, "SAMPLE", "Loading texture from a ZIP", 
		WS_OVERLAPPEDWINDOW | WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, 500, 400, NULL, 
		NULL, hInstance, NULL );

	return hWnd;
}


// ==============================
// WndProc()
//
//	Callback for windows events..
// ==============================
LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_CREATE:
		return 0;
	case WM_DESTROY:
	case WM_CLOSE:
		PostQuitMessage( 0 );
		return 0;
	default: break;
	}

	return DefWindowProc( hWnd, msg, wParam, lParam );
}